import { LCDClient } from "@osmonauts/lcd";
export class LCDQueryClient extends LCDClient {
  constructor({
    restEndpoint
  }) {
    super({
      restEndpoint
    });
  }
  /* Balance queries the balance of a single coin for a single account. */


  async balance(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.denom) !== "undefined") {
      options.params.denom = params.denom;
    }

    const endpoint = `cosmos/bank/v1beta1/balances/${params.address}/by_denom`;
    return await this.request(endpoint, options);
  }
  /* AllBalances queries the balance of all coins for a single account. */


  async allBalances(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    const endpoint = `cosmos/bank/v1beta1/balances/${params.address}`;
    return await this.request(endpoint, options);
  }
  /* SpendableBalances queries the spenable balance of all coins for a single
  account. */


  async spendableBalances(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    const endpoint = `cosmos/bank/v1beta1/spendable_balances/${params.address}`;
    return await this.request(endpoint, options);
  }
  /* TotalSupply queries the total supply of all coins. */


  async totalSupply(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    const endpoint = `cosmos/bank/v1beta1/supply`;
    return await this.request(endpoint, options);
  }
  /* SupplyOf queries the supply of a single coin. */


  async supplyOf(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.denom) !== "undefined") {
      options.params.denom = params.denom;
    }

    const endpoint = `cosmos/bank/v1beta1/supply/by_denom`;
    return await this.request(endpoint, options);
  }
  /* Params queries the parameters of x/bank module. */


  async params(params) {
    const endpoint = `cosmos/bank/v1beta1/params`;
    return await this.request(endpoint);
  }
  /* DenomsMetadata queries the client metadata of a given coin denomination. */


  async denomMetadata(params) {
    const endpoint = `cosmos/bank/v1beta1/denoms_metadata/${params.denom}`;
    return await this.request(endpoint);
  }
  /* DenomsMetadata queries the client metadata for all registered coin
  denominations. */


  async denomsMetadata(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    const endpoint = `cosmos/bank/v1beta1/denoms_metadata`;
    return await this.request(endpoint, options);
  }
  /* DenomOwners queries for all account addresses that own a particular token
  denomination. */


  async denomOwners(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    const endpoint = `cosmos/bank/v1beta1/denom_owners/${params.denom}`;
    return await this.request(endpoint, options);
  }

}