import { LCDClient } from "@osmonauts/lcd";
export class LCDQueryClient extends LCDClient {
  constructor({
    restEndpoint
  }) {
    super({
      restEndpoint
    });
  }
  /* GroupInfo queries group info based on group id. */


  async groupInfo(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.groupId) !== "undefined") {
      options.params.group_id = params.groupId;
    }

    const endpoint = `cosmos/group/v1/group_info/${params.group_id}`;
    return await this.request(endpoint, options);
  }
  /* GroupPolicyInfo queries group policy info based on account address of group policy. */


  async groupPolicyInfo(params) {
    const endpoint = `cosmos/group/v1/group_policy_info/${params.address}`;
    return await this.request(endpoint);
  }
  /* GroupMembers queries members of a group */


  async groupMembers(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.groupId) !== "undefined") {
      options.params.group_id = params.groupId;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    const endpoint = `cosmos/group/v1/group_members/${params.group_id}`;
    return await this.request(endpoint, options);
  }
  /* GroupsByAdmin queries groups by admin address. */


  async groupsByAdmin(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    const endpoint = `cosmos/group/v1/groups_by_admin/${params.admin}`;
    return await this.request(endpoint, options);
  }
  /* GroupPoliciesByGroup queries group policies by group id. */


  async groupPoliciesByGroup(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.groupId) !== "undefined") {
      options.params.group_id = params.groupId;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    const endpoint = `cosmos/group/v1/group_policies_by_group/${params.group_id}`;
    return await this.request(endpoint, options);
  }
  /* GroupsByAdmin queries group policies by admin address. */


  async groupPoliciesByAdmin(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    const endpoint = `cosmos/group/v1/group_policies_by_admin/${params.admin}`;
    return await this.request(endpoint, options);
  }
  /* Proposal queries a proposal based on proposal id. */


  async proposal(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.proposalId) !== "undefined") {
      options.params.proposal_id = params.proposalId;
    }

    const endpoint = `cosmos/group/v1/proposal/${params.proposal_id}`;
    return await this.request(endpoint, options);
  }
  /* ProposalsByGroupPolicy queries proposals based on account address of group policy. */


  async proposalsByGroupPolicy(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    const endpoint = `cosmos/group/v1/proposals_by_group_policy/${params.address}`;
    return await this.request(endpoint, options);
  }
  /* VoteByProposalVoter queries a vote by proposal id and voter. */


  async voteByProposalVoter(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.proposalId) !== "undefined") {
      options.params.proposal_id = params.proposalId;
    }

    const endpoint = `cosmos/group/v1/vote_by_proposal_voter/${params.proposal_id}/${params.voter}`;
    return await this.request(endpoint, options);
  }
  /* VotesByProposal queries a vote by proposal. */


  async votesByProposal(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.proposalId) !== "undefined") {
      options.params.proposal_id = params.proposalId;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    const endpoint = `cosmos/group/v1/votes_by_proposal/${params.proposal_id}`;
    return await this.request(endpoint, options);
  }
  /* VotesByVoter queries a vote by voter. */


  async votesByVoter(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    const endpoint = `cosmos/group/v1/votes_by_voter/${params.voter}`;
    return await this.request(endpoint, options);
  }
  /* GroupsByMember queries groups by member address. */


  async groupsByMember(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    const endpoint = `cosmos/group/v1/groups_by_member/${params.address}`;
    return await this.request(endpoint, options);
  }
  /* TallyResult queries the tally of a proposal votes. */


  async tallyResult(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.proposalId) !== "undefined") {
      options.params.proposal_id = params.proposalId;
    }

    const endpoint = `cosmos/group/v1/proposals/${params.proposal_id}/tally`;
    return await this.request(endpoint, options);
  }

}