import { LCDClient } from "@osmonauts/lcd";
export class LCDQueryClient extends LCDClient {
  constructor({
    restEndpoint
  }) {
    super({
      restEndpoint
    });
  }
  /* Balance queries the number of NFTs of a given class owned by the owner, same as balanceOf in ERC721 */


  async balance(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.classId) !== "undefined") {
      options.params.class_id = params.classId;
    }

    const endpoint = `cosmos/nft/v1beta1/balance/${params.owner}/${params.class_id}`;
    return await this.request(endpoint, options);
  }
  /* Owner queries the owner of the NFT based on its class and id, same as ownerOf in ERC721 */


  async owner(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.classId) !== "undefined") {
      options.params.class_id = params.classId;
    }

    const endpoint = `cosmos/nft/v1beta1/owner/${params.class_id}/${params.id}`;
    return await this.request(endpoint, options);
  }
  /* Supply queries the number of NFTs from the given class, same as totalSupply of ERC721. */


  async supply(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.classId) !== "undefined") {
      options.params.class_id = params.classId;
    }

    const endpoint = `cosmos/nft/v1beta1/supply/${params.class_id}`;
    return await this.request(endpoint, options);
  }
  /* NFTs queries all NFTs of a given class or owner,choose at least one of the two, similar to tokenByIndex in
  ERC721Enumerable */


  async nFTs(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.classId) !== "undefined") {
      options.params.class_id = params.classId;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.owner) !== "undefined") {
      options.params.owner = params.owner;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    const endpoint = `cosmos/nft/v1beta1/nfts`;
    return await this.request(endpoint, options);
  }
  /* NFT queries an NFT based on its class and id. */


  async nFT(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.classId) !== "undefined") {
      options.params.class_id = params.classId;
    }

    const endpoint = `cosmos/nft/v1beta1/nfts/${params.class_id}/${params.id}`;
    return await this.request(endpoint, options);
  }
  /* Class queries an NFT class based on its id */


  async class(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.classId) !== "undefined") {
      options.params.class_id = params.classId;
    }

    const endpoint = `cosmos/nft/v1beta1/classes/${params.class_id}`;
    return await this.request(endpoint, options);
  }
  /* Classes queries all NFT classes */


  async classes(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    const endpoint = `cosmos/nft/v1beta1/classes`;
    return await this.request(endpoint, options);
  }

}