import { LCDClient } from "@osmonauts/lcd";
export class LCDQueryClient extends LCDClient {
  constructor({
    restEndpoint
  }) {
    super({
      restEndpoint
    });
  }
  /* Validators queries all validators that match the given status. */


  async validators(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.status) !== "undefined") {
      options.params.status = params.status;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    const endpoint = `cosmos/staking/v1beta1/validators`;
    return await this.request(endpoint, options);
  }
  /* Validator queries validator info for given validator address. */


  async validator(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.validatorAddr) !== "undefined") {
      options.params.validator_addr = params.validatorAddr;
    }

    const endpoint = `cosmos/staking/v1beta1/validators/${params.validator_addr}`;
    return await this.request(endpoint, options);
  }
  /* ValidatorDelegations queries delegate info for given validator. */


  async validatorDelegations(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.validatorAddr) !== "undefined") {
      options.params.validator_addr = params.validatorAddr;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    const endpoint = `cosmos/staking/v1beta1/validators/${params.validator_addr}/delegations`;
    return await this.request(endpoint, options);
  }
  /* ValidatorUnbondingDelegations queries unbonding delegations of a validator. */


  async validatorUnbondingDelegations(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.validatorAddr) !== "undefined") {
      options.params.validator_addr = params.validatorAddr;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    const endpoint = `cosmos/staking/v1beta1/validators/${params.validator_addr}/unbonding_delegations`;
    return await this.request(endpoint, options);
  }
  /* Delegation queries delegate info for given validator delegator pair. */


  async delegation(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.delegatorAddr) !== "undefined") {
      options.params.delegator_addr = params.delegatorAddr;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.validatorAddr) !== "undefined") {
      options.params.validator_addr = params.validatorAddr;
    }

    const endpoint = `cosmos/staking/v1beta1/validators/${params.validator_addr}delegations/${params.delegator_addr}`;
    return await this.request(endpoint, options);
  }
  /* UnbondingDelegation queries unbonding info for given validator delegator
  pair. */


  async unbondingDelegation(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.delegatorAddr) !== "undefined") {
      options.params.delegator_addr = params.delegatorAddr;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.validatorAddr) !== "undefined") {
      options.params.validator_addr = params.validatorAddr;
    }

    const endpoint = `cosmos/staking/v1beta1/validators/${params.validator_addr}/delegations/${params.delegator_addr}/unbonding_delegation`;
    return await this.request(endpoint, options);
  }
  /* DelegatorDelegations queries all delegations of a given delegator address. */


  async delegatorDelegations(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.delegatorAddr) !== "undefined") {
      options.params.delegator_addr = params.delegatorAddr;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    const endpoint = `cosmos/staking/v1beta1/delegations/${params.delegator_addr}`;
    return await this.request(endpoint, options);
  }
  /* DelegatorUnbondingDelegations queries all unbonding delegations of a given
  delegator address. */


  async delegatorUnbondingDelegations(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.delegatorAddr) !== "undefined") {
      options.params.delegator_addr = params.delegatorAddr;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    const endpoint = `cosmos/staking/v1beta1/delegators/${params.delegator_addr}/unbonding_delegations`;
    return await this.request(endpoint, options);
  }
  /* Redelegations queries redelegations of given address. */


  async redelegations(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.delegatorAddr) !== "undefined") {
      options.params.delegator_addr = params.delegatorAddr;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.srcValidatorAddr) !== "undefined") {
      options.params.src_validator_addr = params.srcValidatorAddr;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.dstValidatorAddr) !== "undefined") {
      options.params.dst_validator_addr = params.dstValidatorAddr;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    const endpoint = `cosmos/staking/v1beta1/delegators/${params.delegator_addr}/redelegations`;
    return await this.request(endpoint, options);
  }
  /* DelegatorValidators queries all validators info for given delegator
  address. */


  async delegatorValidators(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.delegatorAddr) !== "undefined") {
      options.params.delegator_addr = params.delegatorAddr;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    const endpoint = `cosmos/staking/v1beta1/delegators/${params.delegator_addr}/validators`;
    return await this.request(endpoint, options);
  }
  /* DelegatorValidator queries validator info for given delegator validator
  pair. */


  async delegatorValidator(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.delegatorAddr) !== "undefined") {
      options.params.delegator_addr = params.delegatorAddr;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.validatorAddr) !== "undefined") {
      options.params.validator_addr = params.validatorAddr;
    }

    const endpoint = `cosmos/staking/v1beta1/delegators/${params.delegator_addr}validators/${params.validator_addr}`;
    return await this.request(endpoint, options);
  }
  /* HistoricalInfo queries the historical info for given height. */


  async historicalInfo(params) {
    const endpoint = `cosmos/staking/v1beta1/historical_info/${params.height}`;
    return await this.request(endpoint);
  }
  /* Pool queries the pool info. */


  async pool(params) {
    const endpoint = `cosmos/staking/v1beta1/pool`;
    return await this.request(endpoint);
  }
  /* Parameters queries the staking parameters. */


  async params(params) {
    const endpoint = `cosmos/staking/v1beta1/params`;
    return await this.request(endpoint);
  }

}