import { LCDClient } from "@osmonauts/lcd";
export class LCDQueryClient extends LCDClient {
  constructor({
    restEndpoint
  }) {
    super({
      restEndpoint
    });
  }
  /* Channel queries an IBC Channel. */


  async channel(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.portId) !== "undefined") {
      options.params.port_id = params.portId;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.channelId) !== "undefined") {
      options.params.channel_id = params.channelId;
    }

    const endpoint = `ibc/core/channel/v1/channels/${params.channel_id}ports/${params.port_id}`;
    return await this.request(endpoint, options);
  }
  /* Channels queries all the IBC channels of a chain. */


  async channels(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    const endpoint = `ibc/core/channel/v1/channels`;
    return await this.request(endpoint, options);
  }
  /* ConnectionChannels queries all the channels associated with a connection
  end. */


  async connectionChannels(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    const endpoint = `ibc/core/channel/v1/connections/${params.connection}/channels`;
    return await this.request(endpoint, options);
  }
  /* ChannelClientState queries for the client state for the channel associated
  with the provided channel identifiers. */


  async channelClientState(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.portId) !== "undefined") {
      options.params.port_id = params.portId;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.channelId) !== "undefined") {
      options.params.channel_id = params.channelId;
    }

    const endpoint = `ibc/core/channel/v1/channels/${params.channel_id}/ports/${params.port_id}/client_state`;
    return await this.request(endpoint, options);
  }
  /* ChannelConsensusState queries for the consensus state for the channel
  associated with the provided channel identifiers. */


  async channelConsensusState(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.portId) !== "undefined") {
      options.params.port_id = params.portId;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.channelId) !== "undefined") {
      options.params.channel_id = params.channelId;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.revisionNumber) !== "undefined") {
      options.params.revision_number = params.revisionNumber;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.revisionHeight) !== "undefined") {
      options.params.revision_height = params.revisionHeight;
    }

    const endpoint = `ibc/core/channel/v1/channels/${params.channel_id}/ports/${params.port_id}/consensus_state/revision/${params.revision_number}height/${params.revision_height}`;
    return await this.request(endpoint, options);
  }
  /* PacketCommitment queries a stored packet commitment hash. */


  async packetCommitment(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.portId) !== "undefined") {
      options.params.port_id = params.portId;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.channelId) !== "undefined") {
      options.params.channel_id = params.channelId;
    }

    const endpoint = `ibc/core/channel/v1/channels/${params.channel_id}/ports/${params.port_id}packet_commitments/${params.sequence}`;
    return await this.request(endpoint, options);
  }
  /* PacketCommitments returns all the packet commitments hashes associated
  with a channel. */


  async packetCommitments(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.portId) !== "undefined") {
      options.params.port_id = params.portId;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.channelId) !== "undefined") {
      options.params.channel_id = params.channelId;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    const endpoint = `ibc/core/channel/v1/channels/${params.channel_id}/ports/${params.port_id}/packet_commitments`;
    return await this.request(endpoint, options);
  }
  /* PacketReceipt queries if a given packet sequence has been received on the
  queried chain */


  async packetReceipt(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.portId) !== "undefined") {
      options.params.port_id = params.portId;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.channelId) !== "undefined") {
      options.params.channel_id = params.channelId;
    }

    const endpoint = `ibc/core/channel/v1/channels/${params.channel_id}/ports/${params.port_id}packet_receipts/${params.sequence}`;
    return await this.request(endpoint, options);
  }
  /* PacketAcknowledgement queries a stored packet acknowledgement hash. */


  async packetAcknowledgement(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.portId) !== "undefined") {
      options.params.port_id = params.portId;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.channelId) !== "undefined") {
      options.params.channel_id = params.channelId;
    }

    const endpoint = `ibc/core/channel/v1/channels/${params.channel_id}/ports/${params.port_id}packet_acks/${params.sequence}`;
    return await this.request(endpoint, options);
  }
  /* PacketAcknowledgements returns all the packet acknowledgements associated
  with a channel. */


  async packetAcknowledgements(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.portId) !== "undefined") {
      options.params.port_id = params.portId;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.channelId) !== "undefined") {
      options.params.channel_id = params.channelId;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.packetCommitmentSequences) !== "undefined") {
      options.params.packet_commitment_sequences = params.packetCommitmentSequences;
    }

    const endpoint = `ibc/core/channel/v1/channels/${params.channel_id}/ports/${params.port_id}/packet_acknowledgements`;
    return await this.request(endpoint, options);
  }
  /* UnreceivedPackets returns all the unreceived IBC packets associated with a
  channel and sequences. */


  async unreceivedPackets(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.portId) !== "undefined") {
      options.params.port_id = params.portId;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.channelId) !== "undefined") {
      options.params.channel_id = params.channelId;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.packetCommitmentSequences) !== "undefined") {
      options.params.packet_commitment_sequences = params.packetCommitmentSequences;
    }

    const endpoint = `ibc/core/channel/v1/channels/${params.channel_id}/ports/${params.port_id}/packet_commitments/${params.packet_commitment_sequences}/unreceived_packets`;
    return await this.request(endpoint, options);
  }
  /* UnreceivedAcks returns all the unreceived IBC acknowledgements associated
  with a channel and sequences. */


  async unreceivedAcks(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.portId) !== "undefined") {
      options.params.port_id = params.portId;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.channelId) !== "undefined") {
      options.params.channel_id = params.channelId;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.packetAckSequences) !== "undefined") {
      options.params.packet_ack_sequences = params.packetAckSequences;
    }

    const endpoint = `ibc/core/channel/v1/channels/${params.channel_id}/ports/${params.port_id}/packet_commitments/${params.packet_ack_sequences}/unreceived_acks`;
    return await this.request(endpoint, options);
  }
  /* NextSequenceReceive returns the next receive sequence for a given channel. */


  async nextSequenceReceive(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.portId) !== "undefined") {
      options.params.port_id = params.portId;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.channelId) !== "undefined") {
      options.params.channel_id = params.channelId;
    }

    const endpoint = `ibc/core/channel/v1/channels/${params.channel_id}/ports/${params.port_id}/next_sequence`;
    return await this.request(endpoint, options);
  }

}