import { LCDClient } from "@osmonauts/lcd";
export class LCDQueryClient extends LCDClient {
  constructor({
    restEndpoint
  }) {
    super({
      restEndpoint
    });
  }
  /* Connection queries an IBC connection end. */


  async connection(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.connectionId) !== "undefined") {
      options.params.connection_id = params.connectionId;
    }

    const endpoint = `ibc/core/connection/v1/connections/${params.connection_id}`;
    return await this.request(endpoint, options);
  }
  /* Connections queries all the IBC connections of a chain. */


  async connections(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.pagination) !== "undefined") {
      options.params.pagination = params.pagination;
    }

    const endpoint = `ibc/core/connection/v1/connections`;
    return await this.request(endpoint, options);
  }
  /* ClientConnections queries the connection paths associated with a client
  state. */


  async clientConnections(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.clientId) !== "undefined") {
      options.params.client_id = params.clientId;
    }

    const endpoint = `ibc/core/connection/v1/client_connections/${params.client_id}`;
    return await this.request(endpoint, options);
  }
  /* ConnectionClientState queries the client state associated with the
  connection. */


  async connectionClientState(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.connectionId) !== "undefined") {
      options.params.connection_id = params.connectionId;
    }

    const endpoint = `ibc/core/connection/v1/connections/${params.connection_id}/client_state`;
    return await this.request(endpoint, options);
  }
  /* ConnectionConsensusState queries the consensus state associated with the
  connection. */


  async connectionConsensusState(params) {
    const options = {
      params: {}
    };

    if (typeof (params === null || params === void 0 ? void 0 : params.connectionId) !== "undefined") {
      options.params.connection_id = params.connectionId;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.revisionNumber) !== "undefined") {
      options.params.revision_number = params.revisionNumber;
    }

    if (typeof (params === null || params === void 0 ? void 0 : params.revisionHeight) !== "undefined") {
      options.params.revision_height = params.revisionHeight;
    }

    const endpoint = `ibc/core/connection/v1/connections/${params.connection_id}/consensus_state/revision/${params.revision_number}height/${params.revision_height}`;
    return await this.request(endpoint, options);
  }

}